package ircclient

import (
	"bytes"
	"fmt"
	"log"
	"os"
	"testing"
)

func TestFloodTrack(t *testing.T) {
	var Flood FloodTrack
	Flood.Init(3, 2)

	if Flood.Full() {
		t.Error("Expected Track to be empty")
	}

	for x := 1; x < 3; x++ {
		Flood.Save()
		if Flood.Pos != x {
			t.Error(fmt.Sprintf("Expected Track Pos to be %d", x))
		}
		if Flood.Full() {
			t.Error("Expected Track to be empty")
		}
	}

	Flood.Save()
	if Flood.Pos != 3 {
		t.Error("Expected Track Pos to be 3")
	}
	if !Flood.Full() {
		t.Error("Expected Track to be full")
	}
}

func TestThrottleBuffer(t *testing.T) {
	// eat log output
	var logbuff bytes.Buffer
	log.SetOutput(&logbuff)
	defer func() {
		log.SetOutput(os.Stderr)
	}()

	var buff ThrottleBuffer
	buff.init()
	buff.push("#chat", "msg1")
	if !buff.Life_sucks {
		t.Error("Flood control should be enabled here.")
	}
	buff.push("#chat", "msg2")
	buff.push("user", "msg3")

	// verify output order

	var str string
	var expect string
	for _, expect = range []string{"msg1", "msg3", "msg2"} {
		str = buff.pop()
		if str != expect {
			t.Error(fmt.Sprintf("Expected %s, got %s", expect, str))
		}
	}

	if buff.Life_sucks {
		t.Error("Flood control should not be enabled here.")
	}

	// verify deleting 1st item works

	buff.push("#chat", "msg1")
	if !buff.Life_sucks {
		t.Error("Flood control should be enabled here.")
	}
	buff.push("#chat", "msg2")
	buff.push("user", "msg3")
	buff.delete("#chat")

	str = buff.pop()
	expect = "msg3"
	if str != expect {
		t.Error(fmt.Sprintf("Expected %s, got %s", expect, str))
	}

	if buff.Life_sucks {
		t.Error("Flood control should not be enabled here.")
	}

	// verify deleting 2nd item works

	buff.push("#chat", "txt1")
	buff.push("user", "txt2")
	buff.delete("user")
	str = buff.pop()
	expect = "txt1"
	if str != expect {
		t.Error(fmt.Sprintf("Expected %s, got %s", expect, str))
	}

	if buff.Life_sucks {
		t.Error("Flood control should not be enabled here.")
	}

}
